"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
/**
 * Returns the entry in the tree which is closest to the specified input key. The logic determining which entry to
 * locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The entry matching the specified key and operation, or undefined if not found
 */
function find(op, key, tree) {
    var exact = false;
    if (tree._size > 0) {
        switch (op) {
            case 'gte': exact = true;
            case 'gt':
                return internals_1.findMinNodeRightOfKey(exact, key, tree._root, tree._compare);
            case 'lte': exact = true;
            case 'lt':
                return internals_1.findMaxNodeLeftOfKey(exact, key, tree);
            case 'eq':
                return internals_1.findNodeByKey(key, tree);
        }
        throw new Error(`Invalid find operation; must be 'lt', 'lte', 'gt', 'gte' or 'eq'`);
    }
}
exports.find = find;
/**
 * Returns the key of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The key of the matched entry, or undefined if no matching entry was found
 */
function findKey(op, key, tree) {
    const node = find(op, key, tree);
    return core_1.isDefined(node) ? node.key : void 0;
}
exports.findKey = findKey;
/**
 * Returns the value of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(V|undefined)} The value of the matched entry, or undefined if no matching entry was found
 */
function findValue(op, key, tree) {
    const node = find(op, key, tree);
    return core_1.isDefined(node) ? node.value : void 0;
}
exports.findValue = findValue;
/**
 * Returns an iterator pointing at whichever entry in the tree which is closest to the specified input key. The logic
 * determining which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the
 *   matched entry. If no matching entry is found, an empty iterator is returned.
 */
function iterateFrom(op, reverse, key, tree) {
    var exact = false, path;
    if (tree._size > 0) {
        switch (op) {
            case 'gte':
                exact = true;
            case 'gt':
                path = internals_1.findPathToMinNodeRightOfKey(exact, key, tree._root, internals_1.PathNode.NONE, tree._compare);
                break;
            case 'lte':
                exact = true;
            case 'lt':
                path = internals_1.findPathToMaxNodeLeftOfKey(exact, key, tree._root, internals_1.PathNode.NONE, tree._compare);
                break;
            case 'eq':
                path = internals_1.findPathToNodeByKey(key, tree._root, tree._compare);
                break;
            default:
                throw new Error(`Invalid find operation; must be 'lt', 'lte', 'gt', 'gte' or 'eq'`);
        }
    }
    return internals_1.RedBlackTreeIterator.create(core_1.isDefined(path) ? path : internals_1.PathNode.NONE, tree._compare, reverse);
}
exports.iterateFrom = iterateFrom;
function iterateKeysFrom(op, reverse, key, tree) {
    return new internals_1.RedBlackTreeKeyIterator(iterateFrom(op, reverse, key, tree));
}
exports.iterateKeysFrom = iterateKeysFrom;
function iterateValuesFrom(op, reverse, key, tree) {
    return new internals_1.RedBlackTreeValueIterator(iterateFrom(op, reverse, key, tree));
}
exports.iterateValuesFrom = iterateValuesFrom;

//# sourceMappingURL=find.js.map
