"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const core_2 = require("@collectable/core");
const index_1 = require("./index");
const internals_1 = require("../internals");
/**
 * Passes a mutable instance of a tree to a callback function so that batches of changes can be applied without creating
 * additional intermediate copies of the tree, which would waste resources unnecessarily. If the input tree is mutable,
 * it is modified and returned as-is, instead of being cloned beforehand.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {UpdateTreeCallback<K, V>} callback A callback that will be passed a mutable version of the tree
 * @param {RedBlackTreeStructure<K, V>} tree The tree to be updated
 * @returns {RedBlackTreeStructure<K, V>} An updated version of the tree, with changes applied
 */
function updateTree(callback, tree) {
    callback(tree = core_1.Mutation.modify(tree));
    return core_1.Mutation.commit(tree);
}
exports.updateTree = updateTree;
/**
 * Locates a value in the tree and passes it to a callback function that should return an updated value. If the value
 * returned is equal to the old value, then the original tree is returned, otherwise a modified copy of the original
 * tree is returned instead. If the specified key does not exist in the tree, undefined is passed to the callback
 * function, and if a defined value is returned, it is inserted into the tree. If the input tree is mutable, it is
 * modified and returned as-is, instead of being cloned beforehand.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {(UpdateTreeEntryCallback<K, V|undefined>)} callback A callback that will be passed
 * @param {K} key The key of the entry to be updated or inserted
 * @param {RedBlackTreeStructure<K, V>} tree The tree to be updated
 * @returns {RedBlackTreeStructure<K, V>} An updated copy of the tree, or the same tree if the input tree was already mutable
 */
function update(callback, key, tree) {
    var node = internals_1.findNodeByKey(key, tree);
    var oldValue = core_2.isDefined(node) ? node.value : void 0;
    var newValue = callback(oldValue, tree);
    if (core_2.isEqual(oldValue, newValue))
        return tree;
    if (core_2.isDefined(newValue)) {
        tree = index_1.set(key, newValue, tree);
    }
    else {
        tree = index_1.remove(key, tree);
    }
    return tree;
}
exports.update = update;

//# sourceMappingURL=update.js.map
