import { RedBlackTreeStructure, RedBlackTreeIterator, RedBlackTreeKeyIterator, RedBlackTreeValueIterator, RedBlackTreeEntry } from '../internals';
/**
 * An operation used to locate an entry in a tree
 * - "gt": the leftmost entry with a key greater than the specified input key
 * - "gte": the leftmost entry with a key greater than or equal to the specified input key
 * - "lt": the rightmost entry with a key less than the specified input key
 * - "lte": the rightmost entry with a key less than or equal to the specified input key
 * - "eq": the entry for which the key is equal to the specified input key
 */
export declare type FindOp = 'gt' | 'gte' | 'lt' | 'lte' | 'eq';
/**
 * Returns the entry in the tree which is closest to the specified input key. The logic determining which entry to
 * locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The entry matching the specified key and operation, or undefined if not found
 */
export declare function find<K, V = null>(op: FindOp, key: K, tree: RedBlackTreeStructure<K, V>): RedBlackTreeEntry<K, V> | undefined;
/**
 * Returns the key of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The key of the matched entry, or undefined if no matching entry was found
 */
export declare function findKey<K, V = null>(op: FindOp, key: K, tree: RedBlackTreeStructure<K, V>): K | undefined;
/**
 * Returns the value of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(V|undefined)} The value of the matched entry, or undefined if no matching entry was found
 */
export declare function findValue<K, V = null>(op: FindOp, key: K, tree: RedBlackTreeStructure<K, V>): V | undefined;
/**
 * Returns an iterator pointing at whichever entry in the tree which is closest to the specified input key. The logic
 * determining which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the
 *   matched entry. If no matching entry is found, an empty iterator is returned.
 */
export declare function iterateFrom<K, V = null>(op: FindOp, reverse: boolean, key: K, tree: RedBlackTreeStructure<K, V>): RedBlackTreeIterator<K, V>;
export declare function iterateKeysFrom<K, V = null>(op: FindOp, reverse: boolean, key: K, tree: RedBlackTreeStructure<K, V>): RedBlackTreeKeyIterator<K, V>;
export declare function iterateValuesFrom<K, V = null>(op: FindOp, reverse: boolean, key: K, tree: RedBlackTreeStructure<K, V>): RedBlackTreeValueIterator<K, V>;
